#include<stdio.h>
#include<X11/Xlib.h>
#include<GL/glx.h>
#include<GL/glu.h>

void
init_scene (void)
{
	GLfloat ambient[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat specular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat position[] = { 0.0, 1.0, 1.0, 0.0 };

	/* 设置光源 */
	glLightfv (GL_LIGHT0, GL_AMBIENT, ambient);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, diffuse);
	glLightfv (GL_LIGHT0, GL_SPECULAR, specular);
	glLightfv (GL_LIGHT0, GL_POSITION, position);
	glEnable (GL_LIGHTING);
	glEnable (GL_LIGHT0);
	glEnable (GL_AUTO_NORMAL);
	glEnable (GL_NORMALIZE);

	/* 启用深度测试（隐藏面摘除） */
	glEnable (GL_DEPTH_TEST);

	/* 设置投影矩阵 */
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	glOrtho (-1., 1., -1., 1., 1., 20.);

	/* 设置模型视图 */
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	gluLookAt (0., 0., 10., 0., 0., 0., 0., 1., 0.);
}

void
sphere (unsigned int solid, double radius, int slices, int stacks)
{
	GLUquadricObj *quadObj = NULL;
	quadObj = gluNewQuadric ();

	if (solid)
		gluQuadricDrawStyle (quadObj, GLU_FILL);
	else
		gluQuadricDrawStyle (quadObj, GLU_LINE);

	gluQuadricNormals (quadObj, GLU_SMOOTH);
	gluSphere (quadObj, radius, slices, stacks);
}

void
display (void)
{
	/* 背景 */
	glClearColor (0.2, 0.4, 0.6, 1.0);
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	/* 绘制几何体 */
	sphere (1, 0.5f, 100, 100);
}

int
main (int argc, char *argv[])
{
	/* 建立 X 窗口所需的变量 */
	Display *dpy;
	Window root;
	XVisualInfo *vi;
	Colormap cmap;
	XSetWindowAttributes swa;
	Window win;
	XWindowAttributes gwa;
	XEvent xev;
	unsigned int width = 300;
	unsigned int height = 300;

	/* 在 X 窗口中渲染 OpenGL 图形所需变量 */
	GLXFBConfig *fc;
	GLXWindow glw;
	GLXContext glc;
	int att[] = {GLX_RENDER_TYPE, GLX_RGBA_BIT,
		      GLX_DOUBLEBUFFER, True,
		      GLX_DEPTH_SIZE, 16,
		      None};
	int nelements;

	dpy = XOpenDisplay (NULL);
	
	if (dpy == NULL) {
		printf ("\n\tcannot connect to X server\n\n");
		return -1;
	}
	
	fc = glXChooseFBConfig (dpy, 0, att, &nelements);
	
	if (fc == NULL) {
		printf
		    ("\n\tno appropriate framebuffer config found\n\n");
		return -1;
	}
	
	vi = glXGetVisualFromFBConfig (dpy, *fc);
	
	if (vi == NULL) {
		printf ("\n\tno appropriate visual found\n\n");
		return -1;
	}

	root = DefaultRootWindow (dpy);
	cmap = XCreateColormap (dpy, root, vi->visual, AllocNone);

	swa.colormap = cmap;
	swa.event_mask = ExposureMask | KeyPressMask;
	win = XCreateWindow (dpy, root, 0, 0, width, height,
			     0, vi->depth,
			     InputOutput, vi->visual,
			     CWColormap | CWEventMask, &swa);

	XMapWindow (dpy, win);
	XStoreName (dpy, win, "VERY SIMPLE APPLICATION");

	glw = glXCreateWindow (dpy, *fc, win, NULL);
	glc =
	    glXCreateNewContext (dpy, *fc, GLX_RGBA_TYPE, NULL,
				 GL_TRUE);
	glXMakeContextCurrent (dpy, win, win, glc);
	init_scene ();

	while (1) {
		XNextEvent (dpy, &xev);

		if (xev.type == Expose) {
			XGetWindowAttributes (dpy, win, &gwa);
			glViewport (0, 0, gwa.width, gwa.height);
			display ();
			glXSwapBuffers (dpy, win);
		}

		else if (xev.type == KeyPress)
			break;
	}

	glXMakeCurrent (dpy, None, NULL);
	glXDestroyContext (dpy, glc);
	XDestroyWindow (dpy, win);
	XCloseDisplay (dpy);
	
	return 0;
}
